@@ -63,6 +63,8 @@ class FrontendComm(CommBase):
         self.register_call_handler('_send_comm_config',
                                    self._send_comm_config)
 
+        self.comm_lock = threading.Lock()
+
         # self.kernel.parent is IPKernelApp unless we are in tests
         if self.kernel.parent:
             # Create a new socket
@@ -95,6 +97,16 @@ class FrontendComm(CommBase):
 
                 self.kernel.parent.close = close
 
+    def close(self, comm_id=None):
+        """Close the comm and notify the other side."""
+        with self.comm_lock:
+            return super(FrontendComm, self).close(comm_id)
+
+    def _send_message(self, *args, **kwargs):
+        """Publish custom messages to the other side."""
+        with self.comm_lock:
+            return super(FrontendComm, self)._send_message(*args, **kwargs)
+
     def close_thread(self):
         """Close comm."""
         self.comm_thread_close.set()
@@ -128,6 +140,7 @@ class FrontendComm(CommBase):
 
         if msg_type == 'shutdown_request':
             self.comm_thread_close.set()
+            self._comm_close(msg)
             return
 
         handler = self.kernel.shell_handlers.get(msg_type, None)
@@ -208,10 +221,9 @@ class FrontendComm(CommBase):
     def _comm_close(self, msg):
         """Close comm."""
         comm_id = msg['content']['comm_id']
-        comm = self._comms[comm_id]['comm']
-        # Pretend it is already closed to avoid problems when closing
-        comm._closed = True
-        del self._comms[comm_id]
+        # Send back a close message confirmation
+        # Fixes spyder-ide/spyder#15356
+        self.close(comm_id)
 
     def _async_error(self, error_wrapper):
         """
